// ChildFrm.h : interface of the CChildFrame class
//
/////////////////////////////////////////////////////////////////////////////
#pragma once

#include "MDITabCtrl.h"
#include <vector>

#include "MDIChildUserMessenger.h"

#ifdef _DEBUG
	const bool _Donut_ChildFrame_traceOn = true;
	#define dcfTRACE if (_Donut_ChildFrame_traceOn) ATLTRACE
#else
	#define dcfTRACE
#endif

// UDT RAPT [5 buttons mouse] (WinUser.h)
#ifndef WM_APPCOMMAND
#define WM_APPCOMMAND 0x0319
#define FAPPCOMMAND_MASK 0xF000
#define APPCOMMAND_BROWSER_BACKWARD 1
#define APPCOMMAND_BROWSER_FORWARD 2
#define GET_APPCOMMAND_LPARAM(lParam) ((short)(HIWORD(lParam) & ~FAPPCOMMAND_MASK))
#endif
// ENDE

class CChildFrame :
	public CMDIChildWindowImplFixed<CChildFrame, 2>,
	public IWebBrowserEvents2Impl<CChildFrame, ID_DONUTVIEW>,
	public CWebBrowserCommandHandler<CChildFrame>,
	public CUpdateCmdUI<CChildFrame>,
	public CMDIChildUserMessenger<CChildFrame>,
	public CWebBrowser2
{
public:
// Declarations
	DECLARE_FRAME_WND_CLASS(NULL, IDR_MDICHILD)
	DECLARE_FRAME_WND_MENU(NULL)
	typedef CMDIChildWindowImplFixed<CChildFrame, 2> baseClass;

// Constants
	enum { _nPosFavoriteMenuOfTab = 4 };

// Data members
	CDonutView m_view;
	BOOL m_bNavigateBack, m_bNavigateForward;
	CMDITabCtrl& m_MDITab;
	CDonutAddressBar& m_AddressBar;
	bool m_bNewWindow2;
	bool m_bClosing;
	CContainedWindow m_wndBrowser;
	CChildFavoriteMenu<CChildFrame> m_FavMenu;

	CString m_strStatusBar;
	long m_nProgress, m_nProgressMax;

	bool m_bMozilla;

	bool m_bPageFocusInitialized;
	bool m_bWaitNavigateLock;
	HWND m_hWndFocus;
//	CComPtr<IHTMLElement> m_spElm;

// Constructor/Destructor
	CChildFrame(CMDITabCtrl& MDITab, CDonutAddressBar& addressbar,
				bool bNewWindow2, DWORD dwDefaultDLControlFlags)
		: m_bNavigateBack(FALSE), m_bNavigateForward(FALSE), 
		  m_MDITab(MDITab), m_AddressBar(addressbar),
		  m_bNewWindow2(bNewWindow2),
		  m_wndBrowser(this, 7), m_FavMenu(this, ID_INSERTPOINT_CHILDFAVORITEMENU),
		  m_nProgress(-1), m_nProgressMax(0), m_bMozilla(false), m_bPageFocusInitialized(false),
		  m_bClosing(false), m_bWaitNavigateLock(true), m_hWndFocus(NULL), m_view(dwDefaultDLControlFlags)
	{
	}

	~CChildFrame()
	{
		dcfTRACE(_T("CChildFrame::~CChildFrame\n"));

		if (m_wndBrowser.IsWindow()) // scary
			m_wndBrowser.UnsubclassWindow();

		ATLASSERT(m_hMenu == NULL); // no menu
	}

	static CChildFrame* NewWindow(HWND hWndMDIClient, CMDITabCtrl& tabMDI,
		CDonutAddressBar& adBar, bool bNewWindow2 = false, DWORD dwDLFlags = CDLControlOption::s_dwDLControlFlags)
	{
		if (!CMainOption::IsQualify(tabMDI.GetItemCount()))
			return NULL;

		ATLTRY(CChildFrame* pChild = new CChildFrame(tabMDI, adBar, bNewWindow2, dwDLFlags));
		if (pChild != NULL)
			pChild->CreateEx(hWndMDIClient);
		return pChild;
	}

// Overrides
	virtual void OnFinalMessage(HWND /*hWnd*/)
	{
		delete this;
	}

	void ActivateFrame(int nCmdShow = -1)
	{
		ATLASSERT(::IsWindow(m_hWnd));
		m_MDITab.OnMDIChildCreate(m_hWnd);

		if (_check_flag(m_view.m_ViewOption.m_dwExStyle, DVS_EX_OPENNEWWIN)) {
			m_MDITab.NavigateLockTab(m_hWnd, true);
		}

		baseClass::ActivateFrame(nCmdShow);
	}

	void ActivateFrameForNewWindowDelayed(int nCmdShow = -1)
	{
		dcfTRACE(_T("CChildFrame::ActivateFrameForNewWindowDelayed - now activating!\n"));
		if (MDIGetActive() != NULL && CMainOption::s_dwMainExtendedStyle & MAIN_EX_NOACTIVATE_NEWWIN)
			nCmdShow = SW_SHOWNOACTIVATE;

		ActivateFrame(nCmdShow);
	}

// Message map and handlers
	BEGIN_MSG_MAP(CChildFrame)
		PASS_MSG_MAP_MDICHILD_TO_MDITAB(m_MDITab)
		PASS_MSG_MAP_MENUOWNER_TO_EXPMENU(m_FavMenu)

		MESSAGE_HANDLER(WM_SIZE, OnSize)
		MSG_WM_SYSCOMMAND(OnSysCommand)
		MSG_WM_CREATE(OnCreate)
		MSG_WM_CLOSE(OnClose)
		MSG_WM_DESTROY(OnDestroy)
		MSG_WM_MDIACTIVATE(OnMDIActivate)
		MSG_WM_FORWARDMSG(OnForwardMsg)
		MESSAGE_HANDLER(WM_APPCOMMAND, OnAppCommand) // UDT RAPT [5 buttons mouse] (WinUser.h)

		HANDLE_MESSAGE(WM_SETFOCUS)			// baseClass handler is not effective

		USER_MSG_WM_SAVE_OPTION(OnSaveOption)
		USER_MSG_WM_MENU_GOBACK(OnMenuGoBack)		// UDT RAPT (UH) : Travel History
		USER_MSG_WM_MENU_GOFORWARD(OnMenuGoForward)	// UDT RAPT (UH) : Travel History

		MSG_WM_USER_GET_IWEBBROWSER()
		COMMAND_ID_HANDLER_EX(ID_FILE_CLOSE, OnFileClose)
		COMMAND_ID_HANDLER_EX(ID_EDIT_IGNORE, OnEditIgnore)
		COMMAND_ID_HANDLER_EX(ID_EDIT_OPEN_SELECTED_REF, OnEditOpenSelectedRef)
		COMMAND_ID_HANDLER_EX(ID_EDIT_OPEN_SELECTED_TEXT, OnEditOpenSelectedText)

		COMMAND_ID_HANDLER_EX(ID_EDIT_CLOSE_TITLE, OnEditCloseTitle) // UDT DGSTR

		COMMAND_ID_HANDLER_EX(ATL_IDS_SCPREVWINDOW, OnWindowPrev)
		COMMAND_ID_HANDLER_EX(ATL_IDS_SCNEXTWINDOW, OnWindowNext)

		COMMAND_ID_HANDLER_EX(ID_FAVORITE_GROUP_ADD, OnFavoriteGroupAdd)
		COMMAND_ID_HANDLER_EX(ID_FAVORITE_ADD, OnFavoriteAdd)

		COMMAND_ID_HANDLER_EX(ID_VIEW_SETFOCUS, OnViewSetFocus)
		COMMAND_ID_HANDLER_EX(ID_VIEW_REFRESH, OnViewRefresh)
		COMMAND_ID_HANDLER_EX(ID_VIEW_REFRESH_COMPLETELY, OnViewRefreshCompletely) // UDT RAPT
		COMMAND_ID_HANDLER_EX(ID_VIEW_STOP, OnViewStop)
		COMMAND_ID_HANDLER_EX(ID_VIEW_OPTION, OnViewOption)
		COMMAND_ID_HANDLER_EX(ID_FILE_SAVE_AS, OnFileSaveAs)
		COMMAND_ID_HANDLER_EX(ID_FILE_PAGE_SETUP, OnFilePageSetup)
		COMMAND_ID_HANDLER_EX(ID_FILE_PRINT, OnFilePrint)
		COMMAND_ID_HANDLER_EX(ID_FILE_PRINT_PREVIEW, OnFilePrintPreview) // UDT RAPT
		COMMAND_ID_HANDLER_EX(ID_FILE_PROPERTIES, OnFileProperties)

		COMMAND_ID_HANDLER_EX(ID_EDIT_FIND, OnEditFind)

		COMMAND_ID_HANDLER(ID_WINDOW_CLOSE_EXCEPT, OnWindowCloseExcept)

		COMMAND_ID_HANDLER(ID_VIEW_REFRESH_FAVBAR, OnViewRefreshFavBar)

		COMMAND_ID_HANDLER_EX(ID_DOCHOSTUI_OPENNEWWIN, OnDocHostUIOpenNewWin)
		COMMAND_ID_HANDLER_EX(ID_REGISTER_AS_BROWSER, OnRegisterAsBrowser)

		COMMAND_ID_HANDLER_EX(ID_VIEW_BACK1, OnViewBackX)
		COMMAND_ID_HANDLER_EX(ID_VIEW_BACK2, OnViewBackX)
		COMMAND_ID_HANDLER_EX(ID_VIEW_BACK3, OnViewBackX)
		COMMAND_ID_HANDLER_EX(ID_VIEW_BACK4, OnViewBackX)
		COMMAND_ID_HANDLER_EX(ID_VIEW_BACK5, OnViewBackX)
		COMMAND_ID_HANDLER_EX(ID_VIEW_BACK6, OnViewBackX) // UDT RAPT
		COMMAND_ID_HANDLER_EX(ID_VIEW_BACK7, OnViewBackX) // UDT RAPT
		COMMAND_ID_HANDLER_EX(ID_VIEW_BACK8, OnViewBackX) // UDT RAPT
		COMMAND_ID_HANDLER_EX(ID_VIEW_BACK9, OnViewBackX) // UDT RAPT
		COMMAND_ID_HANDLER_EX(ID_VIEW_FORWARD1, OnViewForwardX)
		COMMAND_ID_HANDLER_EX(ID_VIEW_FORWARD2, OnViewForwardX)
		COMMAND_ID_HANDLER_EX(ID_VIEW_FORWARD3, OnViewForwardX)
		COMMAND_ID_HANDLER_EX(ID_VIEW_FORWARD4, OnViewForwardX)
		COMMAND_ID_HANDLER_EX(ID_VIEW_FORWARD5, OnViewForwardX)
		COMMAND_ID_HANDLER_EX(ID_VIEW_FORWARD6, OnViewForwardX) // UDT RAPT
		COMMAND_ID_HANDLER_EX(ID_VIEW_FORWARD7, OnViewForwardX) // UDT RAPT
		COMMAND_ID_HANDLER_EX(ID_VIEW_FORWARD8, OnViewForwardX) // UDT RAPT
		COMMAND_ID_HANDLER_EX(ID_VIEW_FORWARD9, OnViewForwardX) // UDT RAPT

		COMMAND_ID_HANDLER_EX(ID_TITLE_COPY, OnTitleCopy)
		COMMAND_ID_HANDLER_EX(ID_URL_COPY, OnUrlCopy)
		COMMAND_ID_HANDLER_EX(ID_TITLEURL_COPY, OnTitleUrlCopy) // UDT RAPT
		COMMAND_ID_HANDLER_EX(ID_TITLEURL_COPY_MAKEANCHOR, OnTitleUrlCopyMakeAnchor) // UDT RAPT

		CHAIN_COMMANDS(CWebBrowserCommandHandler<CChildFrame>)
		CHAIN_MSG_MAP(CUpdateCmdUI<CChildFrame>)
		CHAIN_MSG_MAP(CMDIChildUserMessenger<CChildFrame>)

		CHAIN_COMMANDS_MEMBER(m_view)// CHAIN_CLIENT_COMMANDS() not send, why?
		CHAIN_MSG_MAP(baseClass)
	ALT_MSG_MAP(7)// to hook WM_CLOSE from Browser internal window (for script: "window.close()")
		MSG_WM_CLOSE(OnBrowserClose)
	END_MSG_MAP()

// Command overrides 

	// UDT RAPT [5 buttons mouse] (WinUser.h)
	LRESULT OnAppCommand(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		if ( !CRaptOption::GetRaptExtendedStyle(RAPT_EX_OPTICALMOUSE) )
			return 0;

		switch ( GET_APPCOMMAND_LPARAM(lParam) ) {
		case APPCOMMAND_BROWSER_BACKWARD:
			GoBack();
			break;
		case APPCOMMAND_BROWSER_FORWARD:
			GoForward();
			break;
		}
		return 0;
	}
	// ENDE

	// UDT RAPT (UH) : Travel History
	LRESULT OnMenuGoBack(HMENU hMenu) {
		MenuChgGoBack(hMenu);
		return 0;
	}
	LRESULT OnMenuGoForward(HMENU hMenu) {
		MenuChgGoForward(hMenu);
		return 0;
	}
	// ENDE

	void OnRegisterAsBrowser(WORD wNotifyCode, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		SetRegisterAsBrowser(wNotifyCode == NM_ON);
	}

	void OnDocHostUIOpenNewWin(WORD wNotifyCode, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		if (wNotifyCode == NM_ON) {
			m_view.m_ViewOption.m_dwExStyle |= DVS_EX_OPENNEWWIN;
			m_MDITab.NavigateLockTab(m_hWnd, true);
		}
		else if (wNotifyCode == NM_OFF) {
			m_view.m_ViewOption.m_dwExStyle &= ~DVS_EX_OPENNEWWIN;
			m_MDITab.NavigateLockTab(m_hWnd, false);
		}
		else if (m_view.m_ViewOption.m_dwExStyle & DVS_EX_OPENNEWWIN) {
			m_view.m_ViewOption.m_dwExStyle &= ~DVS_EX_OPENNEWWIN;
			m_MDITab.NavigateLockTab(m_hWnd, false);
		}
		else {
			m_view.m_ViewOption.m_dwExStyle |= DVS_EX_OPENNEWWIN;
			m_MDITab.NavigateLockTab(m_hWnd, true);
		}
	}

	void OnViewSetFocus(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		// NOTE. _SetPageFoucs would fail if document not completed.
		m_bPageFocusInitialized = false;// reset focus
		_SetPageFocus();
	}

	void OnViewRefresh(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		if (::GetAsyncKeyState(VK_CONTROL) < 0)// Inspired by DOGSTORE, Thanks
			Refresh2(REFRESH_COMPLETELY);
		else
			Refresh();
	}

	// UDT RAPT
	void OnViewRefreshCompletely(WORD, WORD, HWND) {
		Refresh2(REFRESH_COMPLETELY);
	}
	// ENDE

	void OnViewStop(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		Stop();

		// make sure
		m_nDownloadCounter = 0;
		OnStateCompleted(); 
	}

	void OnFavoriteAdd(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		bool bOldShell = _check_flag(MAIN_EX_ADDFAVORITEOLDSHELL, CMainOption::s_dwMainExtendedStyle);
		MtlAddFavorite(GetLocationURL(), MtlGetWindowText(m_hWnd), bOldShell, DonutGetFavoritesFolder(), m_hWnd);
	}

	void OnFavoriteGroupAdd(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		TCHAR szOldPath[MAX_PATH];// save current directory
		::GetCurrentDirectory(MAX_PATH, szOldPath);

		const TCHAR szFilter[] = _T("Donut Favorite Group files(*.dfg)\0*.dfg\0\0");

		::SetCurrentDirectory(DonutGetFavoriteGroupFolder());

		CFileDialog fileDlg(FALSE, _T("dfg"), NULL, OFN_HIDEREADONLY, szFilter);
		fileDlg.m_ofn.lpstrInitialDir = DonutGetFavoriteGroupFolder();
		fileDlg.m_ofn.lpstrTitle = _T("Add to favorites group");
		if (fileDlg.DoModal() == IDOK) {
			_AddGroupOption(fileDlg.m_szFileName);
		}

		// restore current directory
		::SetCurrentDirectory(szOldPath);
	}

	void _AddGroupOption(const CString& strFileName)
	{
		CIniSection pr;
		pr.Open(strFileName, _T("Header"));
		DWORD dwCount = 0;
		pr.QueryValue(dwCount, _T("count"));

		OnSaveOption(strFileName, dwCount);
		dwCount += 1;

		pr.SetValue(dwCount, _T("count"));
//		pr.SetValue(f._nActiveIndex, _T("active"));
		pr.Close();
	}

	void OnClose()
	{
		SetMsgHandled(FALSE);
		m_bClosing = true;

		// for mdi tab ctrl
		HWND hWndActiveNext = NULL;
		if (MDIGetActive() == m_hWnd) {
			int nIndex = m_MDITab.ManageClose(m_hWnd);
			if (nIndex != -1) {
				hWndActiveNext = m_MDITab.GetTabHwnd(nIndex);
				ATLASSERT(::IsWindow(hWndActiveNext));
			}
		}
		m_MDITab.OnMDIChildDestroy(m_hWnd);
		if (hWndActiveNext)
			MDIActivate(hWndActiveNext);

		CString strURL = GetLocationURL();
		if (!strURL.IsEmpty())
			CMainOption::s_pMru->AddToList(strURL);
	}

	LRESULT OnSize(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		dcfTRACE(_T("CChildFrame::OnSize type(%d)\n"), wParam);
		if (CMainOption::s_bTabMode && (wParam == SIZE_RESTORED || wParam == SIZE_MINIMIZED)) {
			dcfTRACE(_T(" restored\n"));
			bHandled = TRUE;// eat it!!
			return 0;
		}
		else {
			bHandled = FALSE;
			return -1;
		}
	}

	void OnSysCommand(UINT uCmdType, CPoint pt)
	{
		if (CMainOption::s_bTabMode && (uCmdType == SC_MINIMIZE || uCmdType == SC_RESTORE))
			SetMsgHandled(TRUE);
		else
			SetMsgHandled(FALSE);
	}

	void OnWindowPrev(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		MDINext(m_hWnd, TRUE);
	}

	void OnWindowNext(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		MDINext(m_hWnd, FALSE);
	}

	LRESULT OnWindowCloseExcept(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		if (!CDonutConfirmOption::OnCloseAllExcept(GetTopLevelParent()))
			return 0;

		CWaitCursor cur;
		CLockRedrawMDIClient lock(m_hWndMDIClient);
		CLockRedraw lock2(m_MDITab);
		MtlCloseAllMDIChildrenExcept(m_hWndMDIClient, m_hWnd);
		return 0;
	}

	LRESULT OnForwardMsg(LPMSG pMsg, DWORD)
	{
		if(baseClass::PreTranslateMessage(pMsg))
			return TRUE;

		return m_view.PreTranslateMessage(pMsg);
	}

	LRESULT OnCreate(LPCREATESTRUCT lpcreatestruct)
	{
		dcfTRACE(_T("CChildFrame::OnCreate\n"));
		// Let me initialize itself
		LRESULT lRet = DefWindowProc();

		m_bMozilla = _bMozilla;// mozilla engine?

		//TODO: Replace with a URL of your choice
		CRect rc = rcDefault;
		__CalcDefaultRect(rc);
		m_hWndClient = m_view.Create(m_hWnd, rc, _T("about:blank"), 
			WS_CHILD | WS_VISIBLE/* | WS_CLIPSIBLINGS | WS_CLIPCHILDREN*/, WS_EX_CLIENTEDGE, ID_DONUTVIEW);

		m_view.QueryControl(IID_IWebBrowser2, (void**)&m_spBrowser);

		WebBrowserEvents2Advise();

		if (_check_flag(MAIN_EX_REGISTER_AS_BROWSER, CMainOption::s_dwMainExtendedStyle))
			SetRegisterAsBrowser(true);// for target name resolution

		_SetupWindowCloseHook();

		// set up for mdi tab ctrl
		CMenuHandle menu0 = m_MDITab.m_menuPopup.GetSubMenu(0);
		HMENU hMenu = menu0.GetSubMenu(_nPosFavoriteMenuOfTab);
		m_FavMenu.InstallExplorerMenu(hMenu);

		if (!m_bNewWindow2)
			SetVisible(true);

		CFavoritesMenuOption::Add(m_hWnd);

		return lRet;
	}

	void __CalcDefaultRect(CRect& rc)
	{
		BOOL bMaximized = FALSE;
		HWND hWndActive = MDIGetActive(&bMaximized);
		if (hWndActive == NULL || bMaximized) {
			::GetClientRect(m_hWndMDIClient, &rc);
		}
	}

	void OnDestroy()
	{
		dcfTRACE(_T("CChildFrame::OnDestroy\n"));
		SetMsgHandled(FALSE);
		m_bClosing = true;

		SetRegisterAsBrowser(false);

		// m_view.DestroyWindow makes offline state reset, so
		// save global offline state
		bool bGlobalOffline = MtlIsGlobalOffline();

		// Before AtlAdviseSinkMap(this, false) destroy view,
		// cuz I wanna handle OnNewWindow2 event
		// And while destroying view, a meaningless WM_DRAWCLIPBOARD may be sent, so ignore it.

		CMainOption::s_bIgnoreUpdateClipboard = true;
		dcfTRACE(_T(" m_view.Destroy\n"));
		if (m_view.IsWindow())
			m_view.DestroyWindow();
		CMainOption::s_bIgnoreUpdateClipboard = false;

		dcfTRACE(_T(" DispEventUnadvise\n"));

		WebBrowserEvents2Unadvise();

		// restore global offline state
		MtlSetGlobalOffline(bGlobalOffline);
	}

	LRESULT OnViewRefreshFavBar(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		m_FavMenu.SetRootDirectoryPath(DonutGetFavoritesFolder());
		m_FavMenu.RefreshMenu();
		return 0;
	}

	void OnFileClose(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		m_bClosing = true;
		PostMessage(WM_CLOSE);
	}

	void OnEditIgnore(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		dcfTRACE(_T("CChildFrame::OnEditIgnore\n"));
		CIgnoredURLsOption::Add(GetLocationURL());
		m_bClosing = true;
		PostMessage(WM_CLOSE);
	}

	// UDT DGSTR ( close Title , but don't post WM_CLOSE so far. )
	void OnEditCloseTitle(WORD , WORD , HWND )
	{
		ATLTRACE2(atlTraceUser, 4, _T("CChildFrame::OnEditCloseTitle\n"));
		CCloseTitlesOption::Add(MtlGetWindowText(m_hWnd));
		//m_bClosing = true;
		//PostMessage(WM_CLOSE);
	}
	// ENDE

	void OnEditOpenSelectedRef(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		dcfTRACE(_T("CChildFrame::OnEditOpenSelectedRef\n"));
		MtlForEachHTMLDocument2(m_spBrowser, _Function_OpenSelectedRef(m_hWnd));
	}

	void OnEditOpenSelectedText(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/)
	{
		dcfTRACE(_T("CChildFrame::OnEditOpenSelectedTitle\n"));
		MtlForEachHTMLDocument2( m_spBrowser, Mtl_bind_mem_fun_void(&CChildFrame::_OpenSelectedText, this) );
	}

	void _OpenSelectedText(IHTMLDocument2* pDocument)
	{
		CComPtr<IHTMLSelectionObject> spSelection;
		HRESULT hr = pDocument->get_selection(&spSelection);
		if (FAILED(hr))
			return;

		CComPtr<IDispatch> spDisp;
		hr = spSelection->createRange(&spDisp);
		if (FAILED(hr))
			return;
		
		CComQIPtr<IHTMLTxtRange> spTxtRange = spDisp;
		if (!spTxtRange)
			return;

		CComBSTR bstrText;
		hr = spTxtRange->get_text(&bstrText);
		if (FAILED(hr) || !bstrText)
			return;

		CString strUrl(bstrText);
		DonutOpenFile(m_hWnd, strUrl);
	}

	struct _Function_OpenSelectedRef
	{
		HWND m_hWnd;
		_Function_OpenSelectedRef(HWND hWnd) : m_hWnd(hWnd) { }

		void operator()(IHTMLDocument2* pDocument)
		{
			CComPtr<IHTMLSelectionObject> spSelection;
			HRESULT hr = pDocument->get_selection(&spSelection);
			if (FAILED(hr))
				return;

			CComPtr<IDispatch> spDisp;
			hr = spSelection->createRange(&spDisp);
			if (FAILED(hr))
				return;
		
			CComQIPtr<IHTMLTxtRange> spTxtRange = spDisp;
			if (!spTxtRange)
				return;

			CComBSTR bstrText;
			hr = spTxtRange->get_htmlText(&bstrText);
			if (FAILED(hr) || !bstrText)
				return;

			CComBSTR bstrLocationUrl;
			hr = pDocument->get_URL(&bstrLocationUrl);
			if (FAILED(hr))
				return;

			dcfTRACE(_T(" Selected:%s\n"), CString(bstrText).Left(200));

			CSimpleArray<CString> arrUrl;
			MtlCreateHrefUrlArray(arrUrl, CString(bstrText), CString(bstrLocationUrl));
			for (int i = 0; i < arrUrl.GetSize(); ++i) {
				CString str(arrUrl[i]);
				Sleep(CRaptOption::s_dwSleepTime); // UDT RAPT
				DonutOpenFile(m_hWnd, arrUrl[i]);
			}
		}
	};

	void OnBrowserClose()
	{
		dcfTRACE(_T("CChildFrame::OnBrowserClose\n"));
		SetMsgHandled(FALSE);
		m_bClosing = true;
		PostMessage(WM_CLOSE);
	}

	void OnSaveOption(LPCTSTR lpszFileName, int nIndex)
	{
		m_view.m_ViewOption.WriteProfile(lpszFileName, nIndex);
	}

	void OnViewBackX(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/)
	{
		int nStep = 0;

		switch (wID) {
		// UDT RAPT
		case ID_VIEW_BACK9:	nStep = 9;	break;
		case ID_VIEW_BACK8:	nStep = 8;	break;
		case ID_VIEW_BACK7:	nStep = 7;	break;
		case ID_VIEW_BACK6:	nStep = 6;	break;
		// ENDE
		case ID_VIEW_BACK5:	nStep = 5;	break;
		case ID_VIEW_BACK4:	nStep = 4;	break;
		case ID_VIEW_BACK3:	nStep = 3;	break;
		case ID_VIEW_BACK2:	nStep = 2;	break;
		case ID_VIEW_BACK1:	nStep = 1;	break;
		default:
			ATLASSERT(FALSE);
		}

//		CLockRedraw lock(m_hWnd);
		for (int i = 0; i < nStep; ++i)
			GoBack();
	}

	void OnViewForwardX(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/)
	{
		int nStep = 0;

		switch (wID) {
		// UDT RAPT
		case ID_VIEW_FORWARD9:	nStep = 9;	break;
		case ID_VIEW_FORWARD8:	nStep = 8;	break;
		case ID_VIEW_FORWARD7:	nStep = 7;	break;
		case ID_VIEW_FORWARD6:	nStep = 6;	break;
		// UDT ENDE
		case ID_VIEW_FORWARD5:	nStep = 5;	break;
		case ID_VIEW_FORWARD4:	nStep = 4;	break;
		case ID_VIEW_FORWARD3:	nStep = 3;	break;
		case ID_VIEW_FORWARD2:	nStep = 2;	break;
		case ID_VIEW_FORWARD1:	nStep = 1;	break;
		default:
			ATLASSERT(FALSE);
		}

//		CLockRedraw lock(m_hWnd);
		for (int i = 0; i < nStep; ++i)
			GoForward();
	}

	void OnTitleCopy(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/)
	{
		MtlSetClipboardText(MtlGetWindowText(m_hWnd), m_hWnd);
	}

	void OnUrlCopy(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/)
	{
		MtlSetClipboardText(GetLocationURL(), m_hWnd);
	}

	// UDT RAPT
	void OnTitleUrlCopy(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/) {
		CString str;
		str.Format(_T("%s %s"), MtlGetWindowText(m_hWnd), GetLocationURL());
		MtlSetClipboardText(str, m_hWnd);
	}
	void OnTitleUrlCopyMakeAnchor(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/) {
		CString str, strTitle(MtlGetWindowText(m_hWnd));
		str.Format(_T("<a href=\"%s\" title=\"Go to %s\">%s</a>"), GetLocationURL(), strTitle, strTitle);
		MtlSetClipboardText(str, m_hWnd);
	}
	// ENDE

	// It will be a template method.
	void OnMDIActivate(HWND hwndChildDeact, HWND hwndChildAct)
	{
		dcfTRACE(_T("CChildFrame::OnMDIActivate\n"));
		SetMsgHandled(FALSE);
			
		if (hwndChildAct == m_hWnd) {	// I'm activated
			m_AddressBar.SetWindowText(GetLocationURL());
			_SetPageFocus();
			_RestoreFocus();
			return;
		}
		
		if (hwndChildDeact == m_hWnd) {		// I'm deactivated
			_SaveFocus();
		}
	}

// Event handlers
	void OnStatusTextChange(const CString& strText)
	{
		m_strStatusBar = strText;

		if (MDIGetActive() == m_hWnd) {
			CWindow wndFrame = GetTopLevelParent();
			CWindow wndStatus = wndFrame.GetDlgItem(ATL_IDW_STATUS_BAR);
			MtlSetWindowText(wndStatus.m_hWnd, strText);
			// UDT DGSTR ( update
			LPCTSTR lpszText = strText;
			SendMessage(GetTopLevelParent(), WM_UPDATE_TITLEBAR, (WPARAM)lpszText, (LPARAM)0);
			// ENDE
		}
	}

	void OnDocumentComplete(IDispatch* pDisp, const CString& strURL)
	{
		if (IsPageIWebBrowser(pDisp)) {	// it means a page complete downloading
			dcfTRACE(_T("CChildFrame::OnDocumentComplete\n"));
			m_bWaitNavigateLock = false;// you are allowed!

			if (MDIGetActive() == m_hWnd && DonutBrowserCanSetFocus(m_hWnd)) {
				_SetPageFocus();
			}
		}
	}

	void OnBeforeNavigate2(IDispatch* pDisp, const CString& strURL, DWORD nFlags,
		const CString& strTargetFrameName, CSimpleArray<BYTE>& baPostedData, const CString& strHeaders, bool& bCancel)
	{
		dcfTRACE(_T("CChildFrame::BeforeNavigate2 url(%s) frame(%s)\n"), strURL.Left(100), strTargetFrameName);

		if (m_bClosing) {
			bCancel = true;
			return;
		}

		// deter popups
		if (m_bNewWindow2 && IsPageIWebBrowser(pDisp) && CIgnoredURLsOption::SearchString(strURL)) {// close ignored url
			bCancel = true;
			m_bNewWindow2 = false;
			m_bClosing = true;
			PostMessage(WM_CLOSE);// It's possible to post twice, but don't care.
			return;
		}

		// Navigate lock
		if (_check_flag(DVS_EX_OPENNEWWIN, m_view.m_ViewOption.m_dwExStyle) &&
			m_bWaitNavigateLock == false &&	!IsRefreshBeforeNavigate2(pDisp))
		{
			dcfTRACE(_T(" Locked, open new window\n"));
			CChildFrame* pChild = CChildFrame::NewWindow(m_hWndMDIClient, m_MDITab, m_AddressBar,
														 false, _GetInheritedDLControlFlags());
			if (pChild) {
				pChild->ActivateFrameForNewWindowDelayed();
				pChild->Navigate2(strURL);
			}
			bCancel = true;
			return;
		}

		// Note: Some ActivateFrame would go mad, because MDI client window not setup yet.
		if (m_bNewWindow2) {// delayed activation if OnNewWindow2 called
			ActivateFrameForNewWindowDelayed();
			m_bNewWindow2 = false;
		}
	}

	void OnProgressChange(long progress, long progressMax)
	{
		m_nProgress = progress;
		m_nProgressMax = progressMax;
	}

	void OnCommandStateChange(long Command, bool bEnable)
	{
		if (Command == CSC_NAVIGATEBACK) {
			m_bNavigateBack = bEnable ? TRUE : FALSE;
		}		
		else if (Command == CSC_NAVIGATEFORWARD) {
			m_bNavigateForward = bEnable ? TRUE : FALSE;
		}
		else if (Command == CSC_UPDATECOMMANDS) {
		}
	}

	void OnDownloadComplete()
	{
		dcfTRACE(_T("CChildFrame::OnDownloadComplete\n"));

		if (m_bMozilla) {
			m_MDITab.SetComplete(m_hWnd);
			// required for mozilla cuz OnTitleChange seems to be not supported yet
			m_nProgress = m_nProgressMax = 0;
			SetWindowText(GetLocationName());

			// change address bar
			if (MDIGetActive() == m_hWnd)
				m_AddressBar.SetWindowText(GetLocationURL());
		}
	}

	void OnTitleChange(const CString& strTitle)
	{
		CString strURL = GetLocationURL();

		if (CCloseTitlesOption::SearchString(strTitle)) {
			m_bClosing = true;
			PostMessage(WM_CLOSE);
			return;
		}

		dcfTRACE(_T("OnTitleChange : %s\n"), strTitle.Left(100));
		// change title
		SetWindowText(MtlCompactString(strTitle, 255));

		// change address bar
		if (MDIGetActive() == m_hWnd)
			m_AddressBar.SetWindowText(strURL);
	}

	void OnNewWindow2(IDispatch** ppDisp, bool& bCancel)
	{
		dcfTRACE(_T("OnNewWindow2\n"));
		if (CMainOption::s_bAppClosing) {
			dcfTRACE(_T(" application closing, ignore this event\n"));
			bCancel = true;
			return;
		}


		CChildFrame* pChild = CChildFrame::NewWindow(m_hWndMDIClient, m_MDITab, m_AddressBar, true, _GetInheritedDLControlFlags());
		// Note: In this case, must be activated in BeforeNavigate2 strangely
		// pChild->ActivateFrameForNewWindowDelayed();
		if (pChild == NULL) {
			bCancel = true;
			return;
		}

		pChild->m_spBrowser->QueryInterface(IID_IDispatch, (void**)ppDisp);
		ATLASSERT(ppDisp != NULL);
	}

	DWORD _GetInheritedDLControlFlags()
	{
		DWORD dwDLFlags;
		if (_check_flag(MAIN_EX_INHERIT_OPTIONS, CMainOption::s_dwMainExtendedStyle))
			dwDLFlags = m_view._GetDLControlFlags();
		else
			dwDLFlags = CDLControlOption::s_dwDLControlFlags;// default

		return dwDLFlags;
	}

	void OnStateConnecting()
	{
		m_MDITab.SetConnecting(m_hWnd);
	}
	void OnStateDownloading()
	{
		m_MDITab.SetDownloading(m_hWnd);
	}
	void OnStateCompleted()
	{
		m_MDITab.SetComplete(m_hWnd);
	}

// Implementation
private:
	void _SaveFocus()
	{
		m_hWndFocus = ::GetFocus();
	}

	void _RestoreFocus()
	{
		if (m_hWndFocus && IsChild(m_hWndFocus) && ::GetFocus() != m_hWndFocus) {
			::SetFocus(m_hWndFocus);
		}
	}

	void _SetPageFocus()
	{
		dcfTRACE(_T("CChildFrame::_SetPageFocus(src:%d)\n"), ::GetFocus());
		if (m_bPageFocusInitialized)
			return;

		if (!MtlIsApplicationActive(m_hWnd))
			return;

		if (IsBrowserNull())
			return;

		CComPtr<IDispatch> spDisp;
		HRESULT hr = m_spBrowser->get_Document(&spDisp);
		if (FAILED(hr))
			return;

		CComQIPtr<IHTMLDocument2> spDoc = spDisp;
		if (!spDoc) // document not initialized yet
			return;

		CComPtr<IHTMLWindow2> spWnd;
		hr = spDoc->get_parentWindow(&spWnd);
		if (!spWnd)
			return;

		m_bPageFocusInitialized = true;	// avoid the endless loop
		hr = spWnd->focus();		// makes mainframe active
	}

	void _SetupWindowCloseHook()
	{
		HWND hWndBrowser;
		::EnumChildWindows(m_view.m_hWnd, (WNDENUMPROC)_EnumProc, (LPARAM)&hWndBrowser);
		ATLASSERT(::IsWindow(hWndBrowser));
		m_wndBrowser.SubclassWindow(hWndBrowser);
	}

	static BOOL CALLBACK _EnumProc(HWND hWnd, LPARAM lParam)
	{
		HWND* phWnd = (HWND*)lParam;
		*phWnd = hWnd;
		return FALSE;
	}

public:
// Update Command UI Map
	BEGIN_UPDATE_COMMAND_UI_MAP(CChildFrame)
		CHAIN_UPDATE_COMMAND_UI_MEMBER(m_view)

		UPDATE_COMMAND_UI_ENABLE_IF_WITH_POPUP(ID_VIEW_BACK, m_bNavigateBack, true)	// with popup
		UPDATE_COMMAND_UI_ENABLE_IF(ID_VIEW_FORWARD, m_bNavigateForward)

		UPDATE_COMMAND_UI_SETDEFAULT_PASS(ID_VIEW_BACK1)
		UPDATE_COMMAND_UI_SETDEFAULT_PASS(ID_VIEW_FORWARD1)

		UPDATE_COMMAND_UI_POPUP_ENABLE_IF(ID_VIEW_BACK1, m_bNavigateBack)	
		UPDATE_COMMAND_UI_POPUP_ENABLE_IF(ID_VIEW_FORWARD1, m_bNavigateForward)
		
		UPDATE_COMMAND_UI(ID_VIEW_FONT_SMALLEST, OnUpdateFontSmallestUI)
		UPDATE_COMMAND_UI(ID_VIEW_FONT_SMALLER, OnUpdateFontSmallerUI)
		UPDATE_COMMAND_UI(ID_VIEW_FONT_MEDIUM, OnUpdateFontMediumUI)
		UPDATE_COMMAND_UI(ID_VIEW_FONT_LARGER, OnUpdateFontLargerUI)
		UPDATE_COMMAND_UI(ID_VIEW_FONT_LARGEST, OnUpdateFontLargestUI)				// with popup

		UPDATE_COMMAND_UI(ID_DEFAULT_PANE, OnUpdateStatusBarUI)
		UPDATE_COMMAND_UI(IDC_PROGRESS, OnUpdateProgressUI)

		UPDATE_COMMAND_UI_ENABLE_IF_WITH_POPUP(ID_WINDOW_RESTORE, _GetShowCmd() != SW_SHOWNORMAL && !CMainOption::s_bTabMode, true)	// with popup
		UPDATE_COMMAND_UI_ENABLE_IF(ID_WINDOW_MOVE, _GetShowCmd() != SW_SHOWMAXIMIZED && !CMainOption::s_bTabMode)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_WINDOW_SIZE, _GetShowCmd() != SW_SHOWMAXIMIZED && _GetShowCmd() != SW_SHOWMINIMIZED && !CMainOption::s_bTabMode)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_WINDOW_MINIMIZE, _GetShowCmd() != SW_SHOWMINIMIZED && !CMainOption::s_bTabMode)
		UPDATE_COMMAND_UI_ENABLE_IF(ID_WINDOW_MAXIMIZE, _GetShowCmd() != SW_SHOWMAXIMIZED && !CMainOption::s_bTabMode)
	END_UPDATE_COMMAND_UI_MAP()

// Update Command UI Handlers
	void OnUpdateFontSmallestUI(CCmdUI* pCmdUI)
	{
		CComVariant var;
		ExecWB(OLECMDID_ZOOM, OLECMDEXECOPT_DONTPROMPTUSER,
			NULL, &var);

		pCmdUI->SetRadio(var == CComVariant(0L) ? true : false);
	}

	void OnUpdateFontSmallerUI(CCmdUI* pCmdUI)
	{
		CComVariant var;
		ExecWB(OLECMDID_ZOOM, OLECMDEXECOPT_DONTPROMPTUSER,
			NULL, &var);

		pCmdUI->SetRadio(var == CComVariant(1L) ? true : false);
	}

	void OnUpdateFontMediumUI(CCmdUI* pCmdUI)
	{
		CComVariant var;
		ExecWB(OLECMDID_ZOOM, OLECMDEXECOPT_DONTPROMPTUSER,
			NULL, &var);

		pCmdUI->SetRadio(var == CComVariant(2L) ? true : false);
	}

	void OnUpdateFontLargerUI(CCmdUI* pCmdUI)
	{
		CComVariant var;
		ExecWB(OLECMDID_ZOOM, OLECMDEXECOPT_DONTPROMPTUSER,
			NULL, &var);

		pCmdUI->SetRadio(var == CComVariant(3L) ? true : false);
	}

	void OnUpdateFontLargestUI(CCmdUI* pCmdUI)
	{
		if (pCmdUI->m_menuSub.m_hMenu) {// popup menu
			pCmdUI->m_menu.EnableMenuItem(pCmdUI->m_nIndex, MF_BYPOSITION | MF_ENABLED);
		}
		else {
			CComVariant var;
			ExecWB(OLECMDID_ZOOM, OLECMDEXECOPT_DONTPROMPTUSER,
				NULL, &var);

			pCmdUI->SetRadio(var == CComVariant(4L) ? true : false);
		}
	}

	void OnUpdateStatusBarUI(CCmdUI* pCmdUI)
	{
		pCmdUI->SetText(m_strStatusBar);
	}

	void OnUpdateProgressUI(CCmdUI* pCmdUI)
	{
		CProgressBarCtrl progressbar = pCmdUI->m_wndOther;
		CStatusBarCtrl statusbar = pCmdUI->m_wndOther.GetParent();

		if (m_nProgress == -1 ||
			(m_nProgress == 0 && m_nProgressMax == 0)) {
			progressbar.ShowWindow(SW_HIDE);
			return;
		}

		CRect rcProgressPart;
		statusbar.GetRect(1, rcProgressPart);
		rcProgressPart.DeflateRect(2, 2);
		progressbar.MoveWindow(rcProgressPart, TRUE);
		progressbar.SetRange32(0, m_nProgressMax);
		progressbar.SetPos(m_nProgress);
		progressbar.ShowWindow(SW_SHOWNORMAL);
	}

	int _GetShowCmd()
	{
		CWindowPlacement wndpl;
		if (m_hWnd && ::GetWindowPlacement(m_hWnd, &wndpl)) {
			return wndpl.showCmd;
		}

		return -1;
	}
};

/////////////////////////////////////////////////////////////////////////////