// SearchBarEditor.h
// by RAPT

#pragma once

#include <atlctrls.h>

#include "RtlDebug.h" // _RTL_DEBUG

// SearchBarExtendedStyle

#define SBR_DEFAULT_EXTENDED_STYLE 0

namespace MTL
{


#define SBR_PANE_STYLE WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | CCS_NODIVIDER | CCS_NOMOVEY



/*
template <class T>
class CSearchBarCommandHandler
{
public:
// Message map and handlers
	BEGIN_MSG_MAP(CSearchBarCommandHandler)
	END_MSG_MAP()
};
*/


class CSearchBarOption
{
public:
// member
	static DWORD s_dwSearchBarExtendedStyle;
	static DWORD s_dwListCountSelect;
	static bool s_bMustResetSearchBar;

// Ctor
	CSearchBarOption()
	{
	}

	static CString GetSearchPath() {
		TCHAR szPathExe[MAX_PATH];
		::GetModuleFileName(_Module.GetModuleInstance(), szPathExe, MAX_PATH);

		CString strFile(szPathExe);
		strFile = strFile.Left(strFile.ReverseFind('\\')+1) + _T("Search.ini");

		return strFile;
	}

// method
	static void SetSearchBarExtendedStyle(DWORD dwMask, bool bOn = true) {
		if (bOn)
			s_dwSearchBarExtendedStyle |= dwMask;
		else
			s_dwSearchBarExtendedStyle &= ~dwMask;
	}

	static bool GetSearchBarExtendedStyle(DWORD dwMask) {
		return (s_dwSearchBarExtendedStyle & dwMask)?true:false;
	}

	static void GetProfile() {
		CString strSearchPath = CSearchBarOption::GetSearchPath();
		CIniSection pr;

		pr.Open(strSearchPath, _T("SearchBar"));
		pr.QueryValue(s_dwListCountSelect, _T("ListCountSelect"));
		pr.Close();
	}

	static void WriteProfile() {
		CString strSearchPath = CSearchBarOption::GetSearchPath();
		CIniSection pr;

		pr.Open(strSearchPath, _T("SearchBar"));
		pr.SetValue(s_dwListCountSelect, _T("ListCountSelect"));
		pr.Close();
	}
};
__declspec(selectany) DWORD CSearchBarOption::s_dwSearchBarExtendedStyle = SBR_DEFAULT_EXTENDED_STYLE;
__declspec(selectany) DWORD CSearchBarOption::s_dwListCountSelect = 0;
__declspec(selectany) bool CSearchBarOption::s_bMustResetSearchBar = false;



class RtlSearchEngine
{
protected:
	typedef std::list<CString> RtlCStringList;
	typedef RtlCStringList::iterator RtlIt;
// member
	RtlCStringList m_arstrNames;
	RtlCStringList m_arstrUrls;
	int m_nSize;

public:
// Ctor
	RtlSearchEngine() : m_nSize(0)
	{
	}

// refer data
	int GetSize() const {
		return m_nSize;
	}

	CString GetAtName(int nIndex) {
		if (nIndex > m_nSize)
			return CString();

		RtlIt it = m_arstrNames.begin();
		std::advance(it, nIndex);
		return CString(*it);
	}

	CString GetAtUrl(int nIndex) {
		if (nIndex > m_nSize)
			return CString();

		RtlIt it = m_arstrUrls.begin();
		std::advance(it, nIndex);
		return CString(*it);
	}

// method
	void Append(CString strName, CString strUrl) {
		m_arstrNames.push_back(strName);
		m_arstrUrls.push_back(strUrl);
		m_nSize++;
	}

	void Clear() {
		m_arstrNames.clear();
		m_arstrUrls.clear();
		m_nSize = 0;
	}

	bool InsertAt(int nIndex, CString strName, CString strUrl) {
		if (nIndex < 0 || nIndex > m_nSize)
			return false; // error
		RtlIt it;

		// Name
		it = m_arstrNames.begin();
		std::advance(it, nIndex);
		m_arstrNames.insert(it, strName);

		// Url
		it = m_arstrUrls.begin();
		std::advance(it, nIndex);
		m_arstrUrls.insert(it, strUrl);

		m_nSize++;
		return true;
	}

	int Find(CString strName) {
		RtlIt it = std::find(m_arstrNames.begin(), m_arstrNames.end(), strName);
		if (it != m_arstrNames.end()) { // found
			int n = 0;
			#ifdef __SGI_STL_INTERNAL_ITERATOR_BASE_H
				std::distance(m_arstrNames.begin(), it, n);
			#else
				n = std::distance(m_arstrNames.begin(), it); // is C++ standard.
			#endif
			return n;
		}
		return -1; // not found
	}

	int Delete(int nIndex) {
		if (nIndex > m_nSize || nIndex < 0)
			return -1; // error : must be 0 .. m_nSize

		RtlIt it;
		// Name
		it = m_arstrNames.begin();
		std::advance(it, nIndex);
		m_arstrNames.erase(it);

		// Url
		it = m_arstrUrls.begin();
		std::advance(it, nIndex);
		m_arstrUrls.erase(it);

		m_nSize--;
		return m_nSize;
	}

	bool Move(int nIndexFrom, int nIndexTo) {
		if (nIndexFrom == nIndexTo)
			return false; // do nothing : no meaning
		if (nIndexFrom >= m_nSize || nIndexFrom < 0)
			return false;
		if (nIndexTo >= m_nSize || nIndexTo < 0)
			return false;

		CString strBuff;
		RtlIt it;

		// Names
		it = m_arstrNames.begin();
		std::advance(it, nIndexFrom);
		strBuff = *it; // copy data
		m_arstrNames.erase(it);

		it = m_arstrNames.begin();
		std::advance(it, nIndexTo);
		m_arstrNames.insert(it, strBuff);

		// Urls
		it = m_arstrUrls.begin();
		std::advance(it, nIndexFrom);
		strBuff = *it; // copy data
		m_arstrUrls.erase(it);

		it = m_arstrUrls.begin();
		std::advance(it, nIndexTo);
		m_arstrUrls.insert(it, strBuff);

		return true;
	}
};



class CSearchBarPropertyPage : public CPropertyPageImpl<CSearchBarPropertyPage>,
	public CWinDataExchange<CSearchBarPropertyPage>
{
public:
// Constants
	enum { IDD = IDD_PROPPAGE_SEARCHBAR };

// Data members
	CString m_strAddressBarName;
	CString m_strAddressBarUrl;
	CCheckListViewCtrl m_listview;
	CEdit m_editName;
	CEdit m_editUrl;
	RtlSearchEngine m_engines;
	TCHAR m_szReplaceChar[2];
	CEdit m_editReplaceChar;
	CString m_strBeforeChange;
	bool m_bIsLoadData;

// Ctor/Dtor
	CSearchBarPropertyPage(const CString& strAddressBarTitle, const CString& strAddressBarURL) : 
		m_strAddressBarName(strAddressBarTitle),
		m_strAddressBarUrl(strAddressBarURL),
		m_strBeforeChange(_T("")),
		m_bIsLoadData(false)
	{
		::lstrcpy(m_szReplaceChar, _T("@"));
		m_engines.Clear(); // reset data
	}
	~CSearchBarPropertyPage()
	{
		if (m_listview.m_hWnd != NULL)
			m_listview.UnsubclassWindow();
	}

// DDX map
	BEGIN_DDX_MAP(CSearchBarPropertyPage)
		DDX_TEXT_LEN(IDC_SBR_EDIT_NAME, m_strAddressBarName, MAX_PATH)
		DDX_TEXT_LEN(IDC_SBR_EDIT_URL, m_strAddressBarUrl, MAX_PATH)
		DDX_TEXT_LEN(IDC_SBR_EDIT_REPLACECHAR, m_szReplaceChar, 1)
	END_DDX_MAP()

// Overrides
	BOOL OnSetActive()
	{
		SetModified(TRUE);
		_SetData();
		return _DoDataExchange(FALSE);
	}
	BOOL OnKillActive()
	{
		return _DoDataExchange(TRUE);
	}
	BOOL OnApply()
	{
		if (_DoDataExchange(TRUE)) {
			_GetData();
			CSearchBarOption::s_bMustResetSearchBar = true; // to be reset SearchBarCategoryCombo
			return TRUE;
		}
		else 
			return FALSE;
	}

// overrides
	BOOL _DoDataExchange(BOOL bSaveAndValidate)// get data from controls?
	{
		if (!bSaveAndValidate) {// set data of control
			if (m_listview.m_hWnd == NULL)
				m_listview.SubclassWindow(GetDlgItem(IDC_SBR_LIST_ENGINE));
			{ // set up column
				const TCHAR* titles[] = { _T("Name"), _T("URL") };
				int widths[] = { 140, 400 };

				LVCOLUMN col;
				col.mask = LVCF_TEXT | LVCF_WIDTH;

				for (int i = 0; i < _countof(titles); i++) {
					col.pszText = (LPTSTR)titles[i];
					col.cx = widths[i];
					m_listview.InsertColumn(i, &col);
				}
			}
			RefreshData(); // reload data

			if (m_editName.m_hWnd == NULL)
				m_editName.Attach(GetDlgItem(IDC_SBR_EDIT_NAME));
			m_editName.SetWindowText(m_strAddressBarName);

			if (m_editUrl.m_hWnd == NULL)
				m_editUrl.Attach(GetDlgItem(IDC_SBR_EDIT_URL));
			m_editUrl.SetWindowText(m_strAddressBarUrl);

			if (m_editReplaceChar.m_hWnd == NULL)
				m_editReplaceChar.Attach(GetDlgItem(IDC_SBR_EDIT_REPLACECHAR));
		}

		return DoDataExchange(bSaveAndValidate);
	}

// Message map and handlers
	BEGIN_MSG_MAP(CSearchBarPropertyPage)
		COMMAND_ID_HANDLER(IDC_URL, OnUrlCmd)
		COMMAND_ID_HANDLER_EX(IDC_SBR_BTN_LOAD, OnButtonLoad)
		COMMAND_ID_HANDLER_EX(IDC_SBR_BTN_EDIT, OnButtonEdit)
		COMMAND_ID_HANDLER_EX(IDC_SBR_BTN_EDITRESET, OnButtonEditReset)
		COMMAND_ID_HANDLER_EX(IDC_SBR_BTN_ADD, OnButtonAdd)
		COMMAND_ID_HANDLER_EX(IDC_SBR_BTN_DEL, OnButtonDel)
		COMMAND_ID_HANDLER_EX(IDC_SBR_BTN_UP, OnButtonMove)
		COMMAND_ID_HANDLER_EX(IDC_SBR_BTN_DOWN, OnButtonMove)
		COMMAND_ID_HANDLER_EX(IDC_SBR_BTN_AUTOREPLACE, OnButtonAutoReplace)
		COMMAND_ID_HANDLER_EX(IDC_SBR_BTN_AUTOREPLACE_REDO, OnButtonAutoReplaceReDo)
		COMMAND_ID_HANDLER_EX(IDC_SBR_BTN_MULTIPUL, OnButtonMultipul)
		CHAIN_MSG_MAP(CPropertyPageImpl<CSearchBarPropertyPage>)
	END_MSG_MAP()

	void OnButtonMultipul(UINT, int, HWND) {
	}

	LRESULT OnUrlCmd(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/) {
		static const TCHAR szFilter[] = _T("Executable File(*.exe)\0*.exe\0Shortcut Files(*.lnk|*.url)\0*.lnk|*.url\0All\0*.*\0\0");

		CFileDialog fileDlg(TRUE, NULL, NULL, 0, szFilter);
		fileDlg.m_ofn.lpstrTitle = _T("Add to search engine list");
		fileDlg.m_ofn.lpstrInitialDir = NULL;
		
		if (fileDlg.DoModal() == IDOK) {
			MtlSetWindowText(m_editUrl, fileDlg.m_szFileName);
		}

		return 0;
	}

	void OnButtonLoad(UINT, int, HWND) {
		CWaitCursor cur;
		_LoadEngineList();
		m_bIsLoadData = true;
		RefreshData();
	}

	void OnButtonEdit(UINT, int, HWND) {
		int nIndex = _GetSelectedItemIndex();
		if (nIndex != LB_ERR) {
			m_editName.SetWindowText(m_engines.GetAtName(nIndex));
			m_editUrl.SetWindowText(m_engines.GetAtUrl(nIndex));
		}
	}

	void OnButtonEditReset(UINT, int, HWND) {
		m_editName.SetWindowText(_T(""));
		m_editUrl.SetWindowText(_T(""));
	}

	void OnButtonAutoReplace(UINT /*wNotifyCode*/, int /*wID*/, HWND /*hWndCtl*/) {
		CString strUrl = MtlGetWindowText(m_editUrl);
		if (strUrl.GetLength() == 0)
			return;

		CString strCh = MtlGetWindowText(m_editReplaceChar);
		if (strCh.GetLength() != 1)
			return;

		if (strUrl.Right(1) == strCh.Left(1)) {
			strUrl.Delete(strUrl.GetLength() - 1);
			m_editUrl.SetWindowText(strUrl);
			return;
		}

		::lstrcpy(m_szReplaceChar, strCh.Left(1));

		RtlString rstr;
		rstr.m_strData = m_strBeforeChange = strUrl;
		CString strFront = rstr.Split(strUrl, 0, m_szReplaceChar[0], false);
		CString strBack = strUrl.Right(strUrl.GetLength() - strFront.GetLength() - 1);
		if (strBack.GetLength() < 1)
			return;

		int nFrontPause = strFront.Find('?');
		if (nFrontPause < 0)
			nFrontPause = strFront.GetLength();
		else
			nFrontPause++;

		if (strBack.Left(1) == '&')
			strBack.Delete(0);
		strBack += '&';
		strFront.Insert(nFrontPause, strBack);
		if (strFront.Right(1) == '&')
			strFront.Delete(strFront.GetLength()-1);

		m_editUrl.SetWindowText(strFront);
	}

	void OnButtonAutoReplaceReDo(UINT, int, HWND) {
		if (m_strBeforeChange.GetLength() < 1)
			return;

		CString strBuff;
		strBuff = MtlGetWindowText(m_editUrl);
		m_editUrl.SetWindowText(m_strBeforeChange);
		m_strBeforeChange = strBuff;
	}

	void OnButtonAdd(UINT /*wNotifyCode*/, int /*wID*/, HWND /*hWndCtl*/) {
		CString strName = MtlGetWindowText(m_editName);
		CString strUrl = MtlGetWindowText(m_editUrl);
		if (strName.GetLength() < 1)
			return;

		if (strUrl.GetLength() < 1)
			strUrl = _T("");

		if (!m_bIsLoadData) {
			int nRet = ::MessageBox(NULL, \
				_T("List data have not loaded.\nTo load list, click 'Load' which up-left on dialog box.\n\nIf you continue this process, data will break.\nDo you want to continue?"), \
				_T("Warning(Search engine list editor)"), MB_YESNO | MB_ICONWARNING | MB_DEFBUTTON2 | MB_TASKMODAL);
			if (nRet == IDCANCEL || nRet == IDNO)
				return;

			m_bIsLoadData = true; // if pressed OK, then never open this window (allowed to write)
		}

		int nIndex = m_engines.Find(strName);
		if (nIndex < 0) { // it's unique

			int nInsertIndex = _GetSelectedItemIndex();
			if (nInsertIndex != LB_ERR) {
				// insert under that point
				m_engines.InsertAt(nInsertIndex + 1, strName, strUrl);
				nIndex = nInsertIndex + 1;
			}
			else {
				// append ( add to the lastest)
				m_engines.Append(strName, strUrl);
			}
			RefreshData();
		}
		_SetSelectedItemIndex(nIndex); // set lsat item
	}

	void OnButtonDel(UINT /*wNotifyCode*/, int /*wID*/, HWND /*hWndCtl*/) {
		int nIndex = _GetSelectedItemIndex();
		if (nIndex == LB_ERR)
			return;

		CString str;
		str.Format(_T("Are you OK to remode this item?\n%s\n%s"), 
			m_engines.GetAtName(nIndex), m_engines.GetAtUrl(nIndex));
		if (IDYES == ::MessageBox(NULL, str, _T("Warning"), MB_YESNO|MB_ICONQUESTION)) {
			m_engines.Delete(nIndex);
			RefreshData();
			_SetSelectedItemIndex(nIndex);
		}
	}

	void OnButtonMove(UINT /*wNotifyCode*/, int wID, HWND /*hWndCtl*/) {
		int nIndex = _GetSelectedItemIndex();
		int nTarget;
		if (nIndex == LB_ERR)
			return;

		switch (wID) {
		case IDC_SBR_BTN_UP:
			nTarget = nIndex - 1;
			if (nTarget < 0)
				nTarget = 0;
			break;

		case IDC_SBR_BTN_DOWN:
			nTarget = nIndex + 1;
			if (nTarget >= m_engines.GetSize())
				nTarget = m_engines.GetSize() - 1;
			break;

		default:
			break;
		}

		m_engines.Move(nIndex, nTarget);
		RefreshData();

		// set selected
		_SetSelectedItemIndex(nTarget);
	}

// Implementation
protected:
	bool _HitTest(CPoint pt) {
		CRect rc;
		m_listview.GetClientRect(rc);
		m_listview.ScreenToClient(&rc);

		if (rc.PtInRect(pt))
			return true;
		else
			return false;
	}

	int _GetSelectedItemIndex() {
		int nCount = m_listview.GetItemCount();

		for (int i = 0; i < nCount; i++) {
			if (m_listview.GetItemState(i, LVIS_SELECTED) == LVIS_SELECTED)
				return i; // selected index : 0 .. (nCount - 1)
		}
		return LB_ERR; // not selected
	}

	void _SetSelectedItemIndex(int _nIndex = -1) {
		int nIndex = _nIndex;
		int nCount = m_listview.GetItemCount();
		if (nIndex >= nCount || nIndex < 0)
			nIndex = nCount - 1;

		m_listview.SetItemState(nIndex, LVIS_SELECTED, LVIS_SELECTED);
	}

	void SetViewData(int nIndex, CString strName, CString strUrl, bool bCheckState = false)
	{
		LVITEM item;
		item.mask = LVIF_TEXT;
		item.iItem = nIndex;

		item.iSubItem = 0;
		item.pszText = (LPTSTR)(LPCTSTR)strName;
		m_listview.InsertItem(&item);

		item.iSubItem = 1;
		item.pszText = (LPTSTR)(LPCTSTR)strUrl;
		m_listview.SetItem(&item);

		m_listview.SetCheckState(nIndex, bCheckState);
	}

	void RefreshData()
	{
		m_listview.DeleteAllItems(); // at 1st, clear data

		// draw data on view
		for (int i = 0; i < m_engines.GetSize(); i++)
			SetViewData(i, m_engines.GetAtName(i), m_engines.GetAtUrl(i));

		// display how many items registed.
		CString str;
		str.Format(_T("%d"), m_engines.GetSize() );
		MtlSetWindowText(GetDlgItem(IDC_SBR_LISTCOUNT), str);
	}

	void _SetData() {
		OnButtonLoad(0, 0, NULL);
	}

	void _GetData()	{
		_SaveEngineList();
	}

	void _LoadEngineList() {
		CString strIniFile(CSearchBarOption::GetSearchPath());
		CIniSection pr;
		DWORD dwValue = 0;
		long lRet;

		// need to clear before use
		m_engines.Clear();

		pr.Open(strIniFile, _T("Search-List"));
		lRet = pr.QueryValue(dwValue, _T("ListCount"));
		pr.Close();

		int nListCount = dwValue;

		if (lRet == ERROR_SUCCESS) {
			CString strKey;
			TCHAR szValue[INTERNET_MAX_PATH_LENGTH];
			DWORD dwCount = INTERNET_MAX_PATH_LENGTH * sizeof(TCHAR);

			for (int i = 1; i <= nListCount; i++) {
				// load name
				strKey.Format(_T("%02d"), i);
				pr.Open(strIniFile, _T("Search-List"));
				lRet = pr.QueryValue(szValue, strKey, &dwCount);
				pr.Close();
				if (lRet != ERROR_SUCCESS)
					break;
				CString strName(szValue);

				// load url
				dwValue = 0;
				lstrcpy(szValue, _T(""));
				pr.Open(strIniFile, strName);
				pr.QueryValue(dwValue, _T("Group"));
				if (dwValue == 0) {
					pr.QueryValue(szValue, _T("URL"), &dwCount);
				} // else, multipul search, it's URL must be blank
				CString strUrl(szValue);

				// append data
				m_engines.Append(strName, strUrl);
			}
		}
	} // _LoadEngineList

	void _SaveEngineList() {
		CString strIniFile(CSearchBarOption::GetSearchPath());
		CString strKey;
		CIniSection pr;
		int nListCount = m_engines.GetSize();

		// write all names
		MtlIniDeleteSection(strIniFile, _T("Search-List")); // clean up
		pr.Open(strIniFile, _T("Search-List"));
		pr.SetValue((DWORD)nListCount, _T("ListCount"));
		for (int i = 1; i <= nListCount; i++) {
			strKey.Format(_T("%02d"), i);
			pr.SetValue(m_engines.GetAtName(i - 1), strKey);
		}
		pr.Close();
		
		// write each urls on each names
		for (i = 0; i < nListCount; i++) {
			pr.Open(strIniFile, m_engines.GetAtName(i));
			CString strUrl(m_engines.GetAtUrl(i));
			if (strUrl.GetLength() > 0) {
				// Group == 0
				pr.SetValue((DWORD)0, _T("Group"));
				pr.SetValue(m_engines.GetAtUrl(i), _T("URL"));
			}
			pr.Close();
		}
	} // _SaveEngineList
};

////////////////////////////////////////////////////////////////////////////
} // namespace MTL;

#ifndef _MTL_NO_AUTOMATIC_NAMESPACE
using namespace MTL;
#endif //!_MTL_NO_AUTOMATIC_NAMESPACE